package org.rcisoft.app.task;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.memInfo.bean.MemberInfoRedisBean;
import org.rcisoft.business.memInfo.dao.MemInfoRepository;
import org.rcisoft.business.memInfo.entity.MemLikeDTO;
import org.rcisoft.core.service.CyRedisService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.Set;

@Component
@Slf4j
@EnableScheduling
public class likeTask {

    @Autowired
    private CyRedisService cyRedisService;

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Autowired
    private MemInfoRepository memInfoRepository;

    /**
     * 用户点赞数据定时任务 将点赞数据同步到对应的表中
     * 会员表 点赞表
     */
    @Scheduled(cron = "0 0 10 * * ?")
    public void userLikeSchedule() {
        //将redis中存储的点赞数同步到opm_user_like表中
        Set<String> keys = redisTemplate.keys(MemberInfoRedisBean.MEMBERINFO_USERLIKE + "*");
        if (keys != null && !keys.isEmpty()) {
            for (String key : keys) {
                //截取key最后一个:后的值，为userId
                int lastIndex = key.lastIndexOf(':');
                Integer userId = Integer.valueOf(key.substring(lastIndex + 1));
                //获取key的所有value
                Map<Object, Object> hmget = cyRedisService.hmget(key);
                if (hmget != null && !hmget.isEmpty()) {
                    //遍历map
                    for (Map.Entry<Object, Object> entry : hmget.entrySet()) {
                        //获取目标
                        String targetId = (String) entry.getKey();
                        //获取点赞数
                        Integer likeCount = (Integer) entry.getValue();
                        //将redis中存储的点赞数同步到opm_user_like表中
                        MemLikeDTO memLikeDTO = new MemLikeDTO();
                        memLikeDTO.setUserId(userId);
                        memLikeDTO.setLikeCount(likeCount);
                        memLikeDTO.setTargetId(Integer.valueOf(targetId));
                        //对点赞表 进行增加点赞数
                        memInfoRepository.addLikeCount(memLikeDTO);
                        //对会员表 进行增加被点赞数
                        memInfoRepository.addMemLikeCount(memLikeDTO);
                    }
                }
            }
        } else {
            System.out.println("No keys found.");
        }
        log.info("用户点赞数据定时任务执行中...");
    }
}
